package net.wanji.opt.cache;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.databus.dao.entity.BaseCrossSchemePO;
import net.wanji.databus.dao.mapper.BaseCrossSchemeMapper;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2023/03/06 14:30
 */
@Component
@Slf4j
public class BaseCrossSchemeInfoCache implements CommandLineRunner {

    @Resource
    BaseCrossSchemeMapper baseCrossSchemeMapper;

    /**
     * key:crossId_id  value:CrossSchemePO
     */
    private static Map<String, BaseCrossSchemePO> crossSchemeInfoMap = new HashMap<>();

    /**
     * 获取所有路口方案信息
     * @return
     */
    public static Map<String, BaseCrossSchemePO> getCrossSchemeInfosCache() {
        return crossSchemeInfoMap;
    }

    /**
     * 通过路口编号_方案id获取方案信息
     * @param key: crossId_id
     * @return
     */
    public BaseCrossSchemePO getSchemePOCache(String key) {
        if (!crossSchemeInfoMap.isEmpty()) {
            return crossSchemeInfoMap.get(key);
        }
        return null;
    }

    /**
     * 刷新缓存
     */
    public void refresh() {
        init();
    }

    @Override
    public void run(String... args) throws Exception {
        init();
    }

    private void init() {

        try {
            long start = System.currentTimeMillis();
            List<BaseCrossSchemePO> baseCrossSchemePOS = baseCrossSchemeMapper.listCrossSchemeInfo(Constants.SystemParam.NULL, Constants.SystemParam.NULL, null);
            if (!CollectionUtils.isEmpty(baseCrossSchemePOS)) {
                baseCrossSchemePOS.forEach(crossSchemePO -> {
                    String crossId = crossSchemePO.getCrossId();
                    Integer schemeId = crossSchemePO.getId();
                    String key = String.join(Constants.SystemParam.SEPARATOR_UNDER_LINE, crossId, String.valueOf(schemeId));
                    crossSchemeInfoMap.put(key, crossSchemePO);
                });
            }

            long end = System.currentTimeMillis();
            log.info("加载路口方案信息到缓存耗时：{}ms,条数据,size={}", (end - start), crossSchemeInfoMap.size());
        } catch (Exception e) {
            log.info("加载路口方案信息到缓存失败", e);
        }
    }
}
