package net.wanji.opt.kafka;

import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2023/03/09 16:03
 */
@Component
@Slf4j
public class ConsumerHandler implements KafkaListenerErrorHandler {

    /**
     * kafka数据缓存
     * key:topic value:data
     */
    private static final Map<String, String> realTimeCarInfoMap = new HashMap<>();

    /**
     * 通过消息主题获取消息信息
     *
     * @param topic
     * @return
     */
    public static String getTopicMessage(String topic) {
        if (realTimeCarInfoMap.isEmpty()) {
            return null;
        }
        return realTimeCarInfoMap.get(topic);
    }

    @KafkaListener(topics = {"cross_real_time_data"})
    public void receiveCrossRealTimeData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) {
        realTimeCarInfoMap.put(record.topic(), String.valueOf(record.value()));
        acknowledgment.acknowledge();
    }
    @KafkaListener(topics = {"cross_real_time_dir_data"})
    public void receiveCrossRealTimeDirData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) {
        realTimeCarInfoMap.put(record.topic(), String.valueOf(record.value()));
        acknowledgment.acknowledge();
    }
    @KafkaListener(topics = {"cross_real_time_turn_data"})
    public void receiveCrossRealTimeTurnData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) {
        realTimeCarInfoMap.put(record.topic(), String.valueOf(record.value()));
        acknowledgment.acknowledge();
    }
    @KafkaListener(topics = {"cross_real_time_lane_data"})
    public void receiveCrossRealTimeLaneData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) {
        realTimeCarInfoMap.put(record.topic(), String.valueOf(record.value()));
        acknowledgment.acknowledge();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

}
