package net.wanji.opt.task;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import net.wanji.common.utils.tool.BeanListUtils;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.*;
import net.wanji.opt.kafka.ConsumerHandler;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/08/17 8:58
 */
@Component
@RequiredArgsConstructor
public class DataCenterProcessTask {

    private final CrossDirDataRealtimeMapper crossDirDataRealTimeMapper;
    private final CrossDirDataHistMapper crossDirDataHistMapper;
    private final CrossTurnDataRealtimeMapper crossTurnDataRealTimeMapper;
    private final CrossTurnDataHistMapper crossTurnDataHistMapper;
    private final CrossLaneDataRealTimeMapper crossLaneDataRealTimeMapper;
    private final CrossLaneDataHistMapper crossLaneDataHistMapper;
    private final CrossDataRealtimeMapper crossDataRealTimeMapper;
    private final CrossDataHistMapper crossDataHistMapper;

    /**
     * @description 收到数据中心kafka中的实时数据插入实时数据表、实时历史表
     * @param
     * @return void
     */

    @Scheduled(fixedRate = 5 * 60 * 1000)
    public void realTimeDataSave() throws Exception {
        ObjectMapper instance = JacksonUtils.getInstance();
        
        String crossRealTimeData = ConsumerHandler.getTopicMessage("cross_real_time_data");
        if (StringUtils.isNotBlank(crossRealTimeData)) {
            List<CrossDataRealtimePO> crossDataRealtimePOS = instance.readValue(crossRealTimeData, new TypeReference<List<CrossDataRealtimePO>>() {});
            if (!CollectionUtils.isEmpty(crossDataRealtimePOS)) {
                crossDataRealTimeMapper.deleteBatch(crossDataRealtimePOS.stream().map(CrossDataRealtimePO::getCrossId).collect(Collectors.toList()));
                crossDataRealTimeMapper.insertBatch(crossDataRealtimePOS);
                List<CrossDataHistPO> crossDataHistPOS = new ArrayList<>(crossDataRealtimePOS.size());
                BeanListUtils.populateList(crossDataRealtimePOS, crossDataHistPOS, CrossDataHistPO.class);
                crossDataHistMapper.insertBatch(crossDataHistPOS);
            }
        }
        
        String crossRealTimeDirData = ConsumerHandler.getTopicMessage("cross_real_time_dir_data");
        if (StringUtils.isNotBlank(crossRealTimeDirData)) {
            List<CrossDirDataRealtimePO> crossDirDataRealtimePOS = instance.readValue(crossRealTimeDirData, new TypeReference<List<CrossDirDataRealtimePO>>() {});
            if (!CollectionUtils.isEmpty(crossDirDataRealtimePOS)) {
                crossDirDataRealTimeMapper.deleteBatch(crossDirDataRealtimePOS.stream().map(CrossDirDataRealtimePO::getCrossId).collect(Collectors.toList()));
                crossDirDataRealTimeMapper.insertBatch(crossDirDataRealtimePOS);
                List<CrossDirDataHistPO> list = new ArrayList<>(crossDirDataRealtimePOS.size());
                BeanListUtils.populateList(crossDirDataRealtimePOS, list, CrossDirDataHistPO.class);
                crossDirDataHistMapper.insertBatch(list);
            }
        }

        String crossRealTimeTurnData = ConsumerHandler.getTopicMessage("cross_real_time_turn_data");
        if (StringUtils.isNotBlank(crossRealTimeTurnData)) {
            List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOS = instance.readValue(crossRealTimeTurnData, new TypeReference<List<CrossTurnDataRealtimePO>>() {});
            if (!CollectionUtils.isEmpty(crossTurnDataRealtimePOS)) {
                crossTurnDataRealTimeMapper.deleteBatch(crossTurnDataRealtimePOS.stream().map(CrossTurnDataRealtimePO::getCrossId).collect(Collectors.toList()));
                crossTurnDataRealTimeMapper.insertBatch(crossTurnDataRealtimePOS);
                List<CrossTurnDataHistPO> list = new ArrayList<>(crossTurnDataRealtimePOS.size());
                BeanListUtils.populateList(crossTurnDataRealtimePOS, list, CrossTurnDataHistPO.class);
                crossTurnDataHistMapper.insertBatch(list);
            }
        }
        
        String crossRealTimeLaneData = ConsumerHandler.getTopicMessage("cross_real_time_lane_data");
        if (StringUtils.isNotBlank(crossRealTimeLaneData)) {
            List<CrossLaneDataRealTimePO> crossLaneDataRealTimePOS = instance.readValue(crossRealTimeLaneData, new TypeReference<List<CrossLaneDataRealTimePO>>() {});
            if (!CollectionUtils.isEmpty(crossLaneDataRealTimePOS)) {
                crossLaneDataRealTimeMapper.deleteBatch(crossLaneDataRealTimePOS.stream().map(CrossLaneDataRealTimePO::getCrossId).collect(Collectors.toList()));
                crossLaneDataRealTimeMapper.insertBatch(crossLaneDataRealTimePOS);
                List<CrossLaneDataHistPO> crossLaneDataHistPOS = new ArrayList<>(crossLaneDataRealTimePOS.size());
                BeanListUtils.populateList(crossLaneDataRealTimePOS, crossLaneDataHistPOS, CrossLaneDataHistPO.class);
                crossLaneDataHistMapper.insertBatch(crossLaneDataHistPOS);
            }
        }
    }
}
