package net.wanji.opt.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/2/9 8:38
 */
@Data
@NoArgsConstructor
@ApiModel(value = "AbnormalCrossListVO", description = "问题路口详情")
public class AbnormalCrossDetailVO {
    @ApiModelProperty(value = "路口状态：0正常；1失衡；2拥堵；3溢出")
    private Integer crossStatus;
    @ApiModelProperty(value = "最大排队")
    private Double crossQueueLength;
    @ApiModelProperty(value = "流率")
    private Double crossFlowRate;
    @ApiModelProperty(value = "平均停车次数")
    private Double crossStopTimes;
    @ApiModelProperty(value = "拥堵指数")
    private Double congestionIndex;
    @ApiModelProperty(value = "转向实时数据")
    private List<TurnDataElement> turnData;
    @ApiModelProperty(value = "方向实时数据")
    private List<DirDataElement> dirData;

    @NoArgsConstructor
    @Data
    public static class TurnDataElement {
        @ApiModelProperty(value = "方向：1北；2东北；3东；4东南；5南；6西南；7西；8西北")
        private Integer dir;
        @ApiModelProperty(value = "转向列表")
        private List<TurnListElement> turnList;

        @NoArgsConstructor
        @Data
        public static class TurnListElement {
            @ApiModelProperty(value = "转向：u掉头；l左转；s直行；r右转")
            private String turn;
            @ApiModelProperty(value = "转向排队长度")
            private Double turnQueueLength;
            @ApiModelProperty(value = "转向平均停车次数")
            private Double turnStopTimes;
            @ApiModelProperty(value = "转向流率 /5min")
            private Integer flow;
        }
    }

    @NoArgsConstructor
    @Data
    public static class DirDataElement {
        @ApiModelProperty(value = "方向：1北；2东北；3东；4东南；5南；6西南；7西；8西北")
        private Integer dir;
        @ApiModelProperty(value = "时间 格式08:12")
        @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "HH:mm", timezone = "GMT+8")
        private Date time;
        @ApiModelProperty(value = "交通状态：1畅通；2缓行；3拥堵；5未知")
        private Integer status;
    }
}
