package net.wanji.opt.vo;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

@NoArgsConstructor
@Data
@ApiModel(value = "GreenwaveDetailVO", description = "子区详情返回值")
public class GreenwaveDetailVO {
    @ApiModelProperty(value = "子区名称")
    private String name;
    @ApiModelProperty(value = "子区状态：1畅通；2缓行；3拥堵；4严重拥堵；5未知")
    private Integer status;
    @ApiModelProperty(value = "非协调相位二次排队")
    private Integer uncoordinatePhaseQueue;
    @ApiModelProperty(value = "协调不停车通过率（%）")
    private Integer noparkPassRate;
    @ApiModelProperty(value = "干线行程时间（秒）")
    private Integer trvalTime;
    @ApiModelProperty(value = "正向绿波带宽（秒）")
    private Integer forwardWidth;
    @ApiModelProperty(value = "反向绿波带宽（秒）")
    private Integer reverseWidth;

    @ApiModelProperty(value = "优化策略列表")
    private List<String> strategyList;
    @ApiModelProperty(value = "选中的优化策略")
    private List<String> strategySelected;

    @ApiModelProperty(value = "子区路口信息")
    List<GreenwaveCross> greenwaveCrossList;

    @ApiModelProperty(value = "正向绿波带")
    private List<GreenwavePoint> forwardGreenwaveList;
    @ApiModelProperty(value = "反向绿波带")
    private List<GreenwavePoint> reverseGreenwaveList;

    @NoArgsConstructor
    @Data
    public static class GreenwaveCross {
        @ApiModelProperty(value = "路口ID")
        private String crossId;
        @ApiModelProperty(value = "路口名称")
        private String crossName;
        @ApiModelProperty(value = "相位差")
        private Integer offset;
        @ApiModelProperty(value = "到下一路口距离")
        private Integer distanceToNextCross;
        @ApiModelProperty(value = "行程速度")
        private Double speed;
        @ApiModelProperty(value = "是否关键路口：0否；1是")
        private Integer isKeyRoute;
        @ApiModelProperty(value = "路口顺序号")
        private Integer sort;

        @ApiModelProperty(value = "绿波相位列表")
        private List<GreenwavePhase> greenwavePhaseList;
    }

    @NoArgsConstructor
    @Data
    public static class GreenwavePhase {
        @ApiModelProperty(value = "相位序")
        private Integer sort;
        @ApiModelProperty(value = "相位号")
        private String phaseNo;
        @ApiModelProperty(value = "相位持续时间（秒）")
        private Integer seconds;
        @ApiModelProperty(value = "是否协调相位 0否 1是")
        private Integer isCoordinate;
    }
    @NoArgsConstructor
    @Data
    public static class GreenwavePoint {
        @ApiModelProperty(value = "路口ID")
        private String crossId;
        @ApiModelProperty(value = "路口名称")
        private String crossName;
        @ApiModelProperty(value = "协调相位时长（秒）")
        private Integer coordSeconds;
        @ApiModelProperty(value = "距离协调相位开始时差（秒）")
        private Integer coordOffset;
        @ApiModelProperty(value = "路口顺序号")
        private Integer sort;
    }
}
