package net.wanji.opt.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/2/9 8:38
 */
@Data
@NoArgsConstructor
@ApiModel(value = "SceneEvaluateabnormalDistributeVO", description = "异常事件时空分布")
public class SceneEvaluateabnormalDistributeVO {
    @ApiModelProperty(value = "拥堵次数")
    Integer congestionTimes;
    @ApiModelProperty(value = "失衡次数")
    Integer unbalanceTimes;
    @ApiModelProperty(value = "溢出次数")
    Integer spilloverTimes;
    @ApiModelProperty(value = "拥堵时长（分钟）")
    Integer congestionDuration;
    @ApiModelProperty(value = "失衡时长（分钟）")
    Integer unbalanceDuration;
    @ApiModelProperty(value = "溢出时长（分钟）")
    Integer spilloverDuration;

    @ApiModelProperty(value = "时间日期分布")
    List<TimeDateDistribution> timeDateDistributionList;

    @NoArgsConstructor
    @Data
    public static class TimeDistribution {

        @ApiModelProperty(value = "日期")
        @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "M/d", timezone = "GMT+8")
        private Date problemDate;

        @ApiModelProperty(value = "状态列表")
        private List<RunningEvaluateMetricsDetailVO.ProblemStatus> problemStatusList;
    }

    @NoArgsConstructor
    @Data
    public static class FinalTimeDistribution {

        @ApiModelProperty(value = "日期")
        @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "M/d", timezone = "GMT+8")
        private Date problemDate;

        @ApiModelProperty(value = "路口状态 0正常 1 失衡  2 拥堵 3 溢出")
        private Integer status;

        @ApiModelProperty(value = "时间")
        @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "HH:mm", timezone = "GMT+8")
        private Date distributionTime;

    }

    @NoArgsConstructor
    @Data
    public static class TimeDateDistribution {

        @ApiModelProperty(value = "时间 HH:mm")
        private String hourMinute;

        @ApiModelProperty(value = "状态列表")
        private List<StatusByDate> statusByDateList;

    }

    @NoArgsConstructor
    @Data
    public static class StatusByDate {

        @ApiModelProperty(value = "日期")
        @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "M/d", timezone = "GMT+8")
        private Date problemDate;

        @ApiModelProperty(value = "是否失衡 0否 1是")
        private Integer isUnbalance = 0;

        @ApiModelProperty(value = "是否溢出 0否 1是")
        private Integer isSpillover = 0;

        @ApiModelProperty(value = "是否拥堵 0否 1是")
        private Integer isCongestion = 0;

    }
}
